'PWB Prefilter v0.2 7/21/2015
'Alan Mandel
'Framingham Public Library

'This script supports Public Web Browser versions 2 and 3
'It processes a file called PreURL.txt and produces PostURL.txt
'Please read ReadMe.txt for details


Option Explicit

Dim GenHttp, SubDomain  'Some constants up front to make building strings easier
Dim BuildStr 'Resulting string for output
Dim FSO  'FileSystem Object
Dim FileInObj, FileOutObj  'Input and output files
Dim OneLine, OrigLine  'One line from the input file
Dim AllDone        'Flag to indicate we should finish up
Dim RejectedEntries  'Running list of entries in the input file that weren't processed
Dim tempStr
Dim HTTPrefix, AllowSubDomains, AllowChar  'Info we'll need to build the output string
Dim OKtoWrite   'True if we are going to write an entry to the output file
Dim CountEntries

AllDone=False
GenHttp = "https?://"
SubDomain = "[A-Za-z0-9\.\-]+\."
CountEntries=0
RejectedEntries = ""

On Error Resume Next

Set FSO = CreateObject("Scripting.FileSystemObject")

'Open input file for reading
Set FileInObj = FSO.OpenTextFile("./PreURL.txt", 1)
If Err.Number<>0 then
   Msgbox("You must have PreURL.txt in the same directory as PreFilter.vbs")
   WScript.Quit
End If

'Open output file for writing
Set FileOutObj = FSO.CreateTextFile("./PostURL.txt", 2, False)
If Err.Number<>0 then
   Msgbox("Cannot create PostURL.txt" & vbnewline & Err.Description & vbnewline & Err.Number)
   WScript.Quit
End If
On Error Goto 0

'Use nested loops here so we can abort an iteration if we want to skip an entry, or when we're ready to write it out

While AllDone=False

   Do While 1
   
      BuildStr = ""
      OKtoWrite=False        'Start off assuming we're not going to write an entry to the output file
      AllowSubDomains=False  'Assume no subdomaining

      'Read a line from input file
      On Error Resume Next
      OneLine = FileInObj.ReadLine()
      If Err.Number<>0 then 
         If Err.Number <> 62 then   '62 should be Input Past End Of File. If we encounter something else, say so
            Msgbox("Error reading input file...aborting" & vbnewline & Err.Description & vbnewline & Err.Number)
         End If
         AllDone=True
         Exit Do
      End If
      On Error Goto 0

      OrigLine=OneLine       'Save the original input line in case we need it later

      If len(OneLine)=0 then Exit Do  'Blank input line

      If left(OneLine,1)="'" then Exit Do   'This line is a comment. Ignore
      
      If left(OneLine,1)=";" then
         BuildStr = OneLine  'Copy comment directly to output file
         OKtoWrite=True
         Exit Do
      End If  


      If len(OneLine)=1 then
         RejectedEntries = RejectedEntries & "Entry too short: " & OrigLine & vbnewline
         Exit Do
      End If

      'Check for pass-thru mode, and if we find it, process it immediately
      If left(OneLine,1)="|" then
         BuildStr = mid(OneLine,2,len(OneLine)-1)
         OKtoWrite=True
         CountEntries=CountEntries+1
         Exit Do
      End If      

      'See if this entry is set to allow the pattern, or reject the pattern
      tempStr = left(OneLine,1)
      if tempStr="+" or tempStr="-" or tempStr="=" then
         AllowChar=tempStr
         'Drop the first character of the line now that we're done with it
         OneLine=mid(OneLine,2,len(OneLine)-1)
      else
         AllowChar="+"
      End If

      
      'Determine which HTTP prefix to use, and then drop the prefix characters from OneLine if necessary
      If StartsWith(OneLine,"http://") then     'HTTP-only
         HTTPrefix="^(http://"
         OneLine = mid(OneLine,8,len(OneLine)-7)
      ElseIf StartsWith(OneLine,"https://") then 'HTTPS-only
         HTTPrefix="^(https://"
         OneLine = mid(OneLine,9,len(OneLine)-8)
      Else  'Allow both HTTP and HTTPS
         HTTPrefix="^(https?://"
      End If

      If StartsWith(OneLine,"*.") then    'Check for allowing subdomains
         AllowSubDomains=True
         OneLine = mid(OneLine,3,len(OneLine)-2)  'Drop the *.
      ElseIf StartsWith(OneLine,"*") then    'Check for allowing subdomains
         AllowSubDomains=True
         OneLine = mid(OneLine,2,len(OneLine)-1)  'Drop the *
      End If

      'Remove any trailing slash to avoid duplication later
      If (len(OneLine)>1) and mid(OneLine,len(OneLine),1)="/" then OneLine = Left(OneLine,Len(OneLine)-1)

      If Len(OneLine)=0 then
         RejectedEntries = RejectedEntries & "Entry too short: " & OrigLine & vbnewline
         Exit Do
      End If

      OneLine = Replace(OneLine,"\","\\")   'Escape any regex characters
      OneLine = Replace(OneLine,"^","\^")
      OneLine = Replace(OneLine,"$","\$")
      OneLine = Replace(OneLine,".","\.")
      OneLine = Replace(OneLine,"?","\?")
      OneLine = Replace(OneLine,"*","\*")
      OneLine = Replace(OneLine,"+","\+")
      OneLine = Replace(OneLine,"(","\(")
      OneLine = Replace(OneLine,")","\)")
      OneLine = Replace(OneLine,"[","\[")
      OneLine = Replace(OneLine,"{","\{")
      OneLine = Replace(OneLine,"|","\|")

      'Construct the output string now using what we have learned from above
      BuildStr = BuildStr & AllowChar & HTTPrefix & OneLine & "/)" & vbnewline
      BuildStr = BuildStr & AllowChar & HTTPrefix & OneLine & "/?)$" & vbnewline
      If AllowSubDomains then
         BuildStr = BuildStr & AllowChar & HTTPrefix & SubDomain & OneLine & "/)" & vbnewline
         BuildStr = BuildStr & AllowChar & HTTPrefix & SubDomain & OneLine & "/?)$" & vbnewline
      End If

      'If we reach here, we're all set to write the entry.      
      OKtoWrite=True
      CountEntries=CountEntries+1
      Exit Do

   Loop

   'If we processed a real entry to write, write it out.
   If OKtoWrite then
      On Error Resume Next
      FileOutObj.WriteLine(BuildStr)
      If Err.Number<>0 then
          Msgbox("Error writing PostURL.txt" & vbnewline & Err.Description & vbnewline & Err.Number)
      End If
      On Error Goto 0
   End If

   'Go back and process another line from the input file (unless done)
Wend

'Finished with the input file
'Let the user know about any lines of significance that we skipped over
TempStr = "All Done - processed " & CountEntries & " filter entries"
If Len(RejectedEntries) = 0 then
   Msgbox(TempStr)
Else
   Msgbox(Tempstr & " with the following skipped entries:" & vbnewline & RejectedEntries)
End If

Wscript.Quit


Function StartsWith(TheString,ToFind)
'Case-insensitive search for beginning of string
Dim SubString

StartsWith=False   'Assume no match

If Len(ToFind)<=Len(TheString) then   'i.e., only search if searchstring isn't longer than main string
  SubString = mid(TheString,1,len(ToFind))
  If LCase(SubString) = LCase(ToFind) then StartsWith=True
End If

End Function